@@ -1603,7 +1603,7 @@ checkSpctVersion <- function(x) {
 #' @family multiple.wl attribute functions
 #'
 setMultipleWl <- function(x, multiple.wl = NULL) {
-  stopifnot(is.any_spct(x))
+  stopifnot(is.any_spct(x) || is.any_summary_spct(x))
   name <- substitute(x)
   if (is.null(multiple.wl)) {
     multiple.wl <- getMultipleWl(x)
@@ -1636,7 +1636,7 @@ setMultipleWl <- function(x, multiple.wl = NULL) {
 #' getMultipleWl(sun.spct)
 #'
 getMultipleWl <- function(x) {
-  if (is.any_spct(x)) {
+  if (is.any_spct(x) || is.any_summary_spct(x)) {
     multiple.wl <- attr(x, "multiple.wl", exact = TRUE)
     if (is.null(multiple.wl) || is.na(multiple.wl) || !is.numeric(multiple.wl)) {
       # need to handle objects created with old versions
@@ -1742,7 +1742,7 @@ setWhenMeasured.generic_mspct <-
       when <- lubridate::with_tz(when.measured, "UTC")
       x <- msmsply(mspct = x, .fun = setWhenMeasured, when.measured = when)
     } else if (length(when.measured) == length(x)) {
-      for (i in 1:length(x)) {
+      for (i in seq_along(x)) {
         when <- when.measured[[i]]
         stopifnot(lubridate::is.POSIXct(when))
         when <- lubridate::with_tz(when, "UTC")
@@ -1938,16 +1938,16 @@ setWhereMeasured.generic_mspct <- function(x,
                  lon = lon)
   } else if (!is.na(where.measured) && !is.data.frame(where.measured) &&
              is.list(where.measured) && length(where.measured) == length(x)) {
-    for (i in 1:length(x)) {
+    for (i in seq_along(x)) {
       x[[i]] <- setWhereMeasured(x[[i]], where.measured = where.measured[[i]])
     }
   } else if (!is.na(where.measured) && is.data.frame(where.measured) &&
              nrow(where.measured) == length(x)) {
-    for (i in 1:length(x)) {
+    for (i in seq_along(x)) {
       x[[i]] <- setWhereMeasured(x[[i]], where.measured = where.measured[i, ])
     }
   } else if (is.na(where.measured) && length(lat) == length(x) && length(lon) == length(x)) {
-    for (i in 1:length(x)) {
+    for (i in seq_along(x)) {
       x[[i]] <- setWhereMeasured(x[[i]], lon = lon[i], lat = lat[i])
     }
   } else {
@@ -2147,7 +2147,7 @@ getInstrSettings <- function(x) {
 #'
 setWhatMeasured <- function(x, what.measured) {
   name <- substitute(x)
-  if (is.any_spct(x)) {
+  if (is.any_spct(x) || is.any_summary_spct(x)) {
     attr(x, "what.measured") <- what.measured
     if (is.name(name)) {
       name <- as.character(name)
@@ -2171,7 +2171,7 @@ setWhatMeasured <- function(x, what.measured) {
 #' @family measurement metadata functions
 #'
 getWhatMeasured <- function(x) {
-  if (is.any_spct(x)) {
+  if (is.any_spct(x) || is.any_summary_spct(x)) {
     what.measured <- attr(x, "what.measured", exact = TRUE)
     if (is.null(what.measured) || is.na(what.measured)) {
       # need to handle objects created with old versions
